from .new_chart_dialog import NewChartDialog
from .color_palette_dialog import ColorPaletteDialog
from .settings_dialog import SettingsDialog
from .stitch_dialog import StitchDialog
from .background_image_dialog import BackgroundImageDialog
from .multiple_stitch_dialog import MultipleStitchDialog

__all__ = [
    'NewChartDialog',
    'ColorPaletteDialog',
    'SettingsDialog',
    'StitchDialog',
    'BackgroundImageDialog',
    'MultipleStitchDialog'
]
